@@ -11,11 +11,11 @@ App::Cmd::Subdispatch::DashedStyle - "app cmd --subcmd" style subdispatching
 
 =head1 VERSION
 
-version 0.013
+version 0.200
 
 =cut
 
-our $VERSION = '0.013';
+our $VERSION = '0.200';
 
 =head1 METHODS
 
@@ -44,6 +44,14 @@ sub get_command {
   }
 }
 
+=begin Pod::Coverage
+
+prepare_default_command
+
+=end Pod::Coverage
+
+=cut
+
 sub prepare_default_command {
   my ( $self, $opt, @args ) = @_;
   $self->_prepare_command( "help" );
